clear;
close all;
clc;

% add path to your Winterc-grav directory
addpath('<directory to your WINTERC-grav model>')
% addpath to your GSH Tools directory
addpath(['<directory to your GSH package>' 'Tools/'])

%%

height = 0;
SHbounds = [4 359];
latLim =   [-89.75 89.75 0.5];
lonLim =   [0.25 359.75 0.5];

% load external datasets
load('WINTERCgrav_geoid_data.mat')
Vg = load('XGM2016.txt');

%%
tic;
[CONTINENTAL,Lon,Lat] = gmt2matrix(importdata('ETOPO2_km_continental.xyz'));
ICE = gmt2matrix(importdata('ETOPO2_km_depth_Ice.xyz'));
BEDROCK = gmt2matrix(importdata('ETOPO2_km_depth_Bed.xyz'));
MOHO = gmt2matrix(importdata('Global_Moho_CRUST1.0.xyz'));

%% LAB is shallower than 100 km
D20_LAY = -BEDROCK; 
D20_LAY(-BEDROCK>-20) = -20;                                                                                                                                       
D20_LAY(-MOHO>-20) = -MOHO(-MOHO>-20); 

D40_LAY = -BEDROCK; 
D40_LAY(-BEDROCK>-40) = -40;                                                                                                                                       
D40_LAY(-MOHO>-40) = -MOHO(-MOHO>-40); 

D60_LAY = -BEDROCK; 
D60_LAY(-BEDROCK>-60) = -60;                                                                                                                                       
D60_LAY(-MOHO>-60) = -MOHO(-MOHO>-60); 

%%
L20KM = -MOHO;
L20KM(L20KM>=-20) = -20;

L36KM = -MOHO;
L36KM(L36KM>=-36) = -36;

L56KM = -MOHO;
L56KM(L56KM>=-56) = -56;

L80KM = -MOHO;
L80KM(L80KM>=-80) = -80;

%%
RCRUST = gmt2matrix(importdata('rho_c_out.xyz'));

Model = struct();

Model.number_of_layers = 16;
Model.name = 'BenchmarkWgrav';

% Additional variables

Model.GM = 3.9860004415E14;    
Model.Re = 6378136.30;
Model.geoid = 'none';
Model.nmax = 359;                 

% % water layer
Model.l1.bound = -CONTINENTAL.*1e3;
Model.l1.dens  = 1030;
Model.l1.alpha = 0;

% ice layer
Model.l2.bound = -ICE.*1e3;
Model.l2.dens = 910;
Model.l2.alpha = 0;

% crustal layer1
Model.l3.bound = -BEDROCK.*1e3;
Model.l3.dens = RCRUST;
Model.l3.alpha = 0;

% crustal layer2
Model.l4.bound = D20_LAY.*1e3;
Model.l4.dens = RCRUST;
Model.l4.alpha = 0;

% crustal layer3
Model.l5.bound = D40_LAY.*1e3;
Model.l5.dens = RCRUST;
Model.l5.alpha = 0;

% crustal layer4
Model.l6.bound = D60_LAY.*1e3;
Model.l6.dens = RCRUST;
Model.l6.alpha = 0;

% subMoho-20km
Model.l7.bound = -MOHO.*1e3;
[rho0, alpha] = determineGradient(gmt2matrix(importdata('rho_submoho_out.xyz'))...
                                 ,gmt2matrix(importdata('rho_20km_out.xyz'))...
                                 ,-MOHO.*1e3,-L20KM.*1e3,Model.Re);
Model.l7.dens = rho0;
Model.l7.alpha = alpha;

% 20km-36km
Model.l8.bound = L20KM.*1e3;
[rho0, alpha] = determineGradient(gmt2matrix(importdata('rho_20km_out.xyz'))...
                                 ,gmt2matrix(importdata('rho_36km_out.xyz'))...
                                 ,-L20KM.*1e3,-L36KM.*1e3,Model.Re);
Model.l8.dens = rho0;
Model.l8.alpha = alpha;

% 36km-56km
Model.l9.bound = L36KM.*1e3;
[rho0, alpha] = determineGradient(gmt2matrix(importdata('rho_36km_out.xyz'))...
                                 ,gmt2matrix(importdata('rho_56km_out.xyz'))...
                                 ,-L36KM.*1e3,-L56KM.*1e3,Model.Re);
Model.l9.dens = rho0;
Model.l9.alpha = alpha;

% 56km-80km
Model.l10.bound = L56KM.*1e3;
[rho0, alpha] = determineGradient(gmt2matrix(importdata('rho_56km_out.xyz'))...
                                 ,gmt2matrix(importdata('rho_80km_out.xyz'))...
                                 ,-L56KM.*1e3,-L80KM.*1e3,Model.Re);
Model.l10.dens = rho0;
Model.l10.alpha = alpha;

% 80km-110km
Model.l11.bound = L80KM.*1e3;
[rho0, alpha] = determineGradient(gmt2matrix(importdata('rho_80km_out.xyz'))...
                                 ,gmt2matrix(importdata('rho_110km_out.xyz'))...
                                 ,-L80KM.*1e3,-110000.*ones(size(L80KM)),Model.Re);
Model.l11.dens = rho0;
Model.l11.alpha = alpha;

% 110km-150km
Model.l12.bound = -110000.*ones(size(L80KM));
[rho0, alpha] = determineGradient(gmt2matrix(importdata('rho_110km_out.xyz'))...
                                 ,gmt2matrix(importdata('rho_150km_out.xyz'))...
                                 ,-110000.*ones(size(L80KM)),-150000.*ones(size(L80KM)),Model.Re);
Model.l12.dens = rho0;
Model.l12.alpha = alpha;

% 150km-200km
Model.l13.bound = -150000.*ones(size(L80KM));
[rho0, alpha] = determineGradient(gmt2matrix(importdata('rho_150km_out.xyz'))...
                                 ,gmt2matrix(importdata('rho_200km_out.xyz'))...
                                 ,-150000.*ones(size(L80KM)),-200000.*ones(size(L80KM)),Model.Re);
Model.l13.dens = rho0;
Model.l13.alpha = alpha;

% 200km-260km
Model.l14.bound = -200000.*ones(size(L80KM));
[rho0, alpha] = determineGradient(gmt2matrix(importdata('rho_200km_out.xyz'))...
                                 ,gmt2matrix(importdata('rho_260km_out.xyz'))...
                                 ,-200000.*ones(size(L80KM)),-260000.*ones(size(L80KM)),Model.Re);
Model.l14.dens = rho0;
Model.l14.alpha = alpha;

% 260km-330km
Model.l15.bound = -260000.*ones(size(L80KM));
[rho0, alpha] = determineGradient(gmt2matrix(importdata('rho_260km_out.xyz'))...
                                 ,gmt2matrix(importdata('rho_330km_out.xyz'))...
                                 ,-260000.*ones(size(L80KM)),-330000.*ones(size(L80KM)),Model.Re);
Model.l15.dens = rho0;
Model.l15.alpha = alpha;

% 330km-400km
Model.l16.bound = -330000.*ones(size(L80KM));
[rho0, alpha] = determineGradient(gmt2matrix(importdata('rho_330km_out.xyz'))...
                                 ,gmt2matrix(importdata('rho_400km_out.xyz'))...
                                 ,-330000.*ones(size(L80KM)),-400000.*ones(size(L80KM)),Model.Re);
Model.l16.dens = rho0;
Model.l16.alpha = alpha;

Model.l17.bound = -400000.*ones(size(L80KM));

%% analyses

[V] = model_SH_analysis(Model);
toc
%% synthesis
tic;

disp('Constructing the synthesis')

[data] = model_SH_synthesis(lonLim,latLim,height,SHbounds,V,Model);
toc
%% Gravity observations the publically available XGM2016 model

Vg(1,3) = Vg(1,3) - 1.000000000000000;
Vg(3,3) = Vg(3,3) + 4.841668548961195e-04;
Vg(5,3) = Vg(5,3) - 7.903040733333333e-07;
Vg(7,3) = Vg(7,3) + 1.687251001365147e-09;
Vg(9,3) = Vg(9,3) - 3.460983369268472e-12;
Vg(Vg(:,1)>SHbounds(2),:) = [];
% 
[datag] = model_SH_synthesis(lonLim,latLim,height,SHbounds,Vg,Model);

%%
gamma = Model.GM/Model.Re^2;

%%
lon = data.grd.lon(1,:);
lats = data.grd.lat(:,1);
coast = load('coast.mat');

%%
gamma = 9.81;

figure
subplot(2,2,1)
imagesc(lon,lats,(flipud(GT)));c=colorbar;colormap(parula)
hold on
plot(coast.long,coast.lat,'k','LineWidth',1.5);plot(coast.long+360,coast.lat,'k','LineWidth',1.5);
xlim([min(lon) max(lon)])
ylim([min(lats) max(lats)])
hold off
caxis([-100 100])
xlabel('Longitude [^o]','FontSize',20)
ylabel('Latitude [^o]','FontSize',20)
title(['Geoid WINTERC'],'FontSize',20)
ylabel(c,'m','FontSize',20) 
set(gca,'FontSize',20,'YDir','normal')

subplot(2,2,2)
imagesc(lon,lats,(flipud(GT)-(data.pot)./gamma));c=colorbar;colormap(parula)
hold on
plot(coast.long,coast.lat,'k','LineWidth',1.5);plot(coast.long+360,coast.lat,'k','LineWidth',1.5);
xlim([min(lon) max(lon)])
ylim([min(lats) max(lats)])
hold off
%thres = max(max(abs(flipud(GT)-(data.pot)./gamma)));
thres = max(max(abs((data.pot-datag.pot)./gamma)));
caxis([-thres thres])
%caxis([-.2 0.2])
xlabel('Longitude [^o]','FontSize',20)
ylabel('Latitude [^o]','FontSize',20)
title(['WINTERC - GSH'],'FontSize',20)
ylabel(c,'m','FontSize',20) 
set(gca,'FontSize',20,'YDir','normal')

subplot(2,2,3)
imagesc(lon,lats,((data.pot-datag.pot)./gamma));c=colorbar;colormap(parula)
hold on
plot(coast.long,coast.lat,'k','LineWidth',1.5);plot(coast.long+360,coast.lat,'k','LineWidth',1.5);
xlim([min(lon) max(lon)])
ylim([min(lats) max(lats)])
hold off
%thres = max(max(abs((data.pot-datag.pot)./gamma)));
caxis([-thres thres])
%caxis([-1 1])
xlabel('Longitude [^o]','FontSize',20)
ylabel('Latitude [^o]','FontSize',20)
title(['GSH - XGM2016'],'FontSize',20)
ylabel(c,'m','FontSize',20) 
set(gca,'FontSize',20,'YDir','normal')

subplot(2,2,4)
imagesc(lon,lats,(flipud(GT)-(datag.pot)./gamma));c=colorbar;colormap(parula)
hold on
plot(coast.long,coast.lat,'k','LineWidth',1.5);plot(coast.long+360,coast.lat,'k','LineWidth',1.5);
xlim([min(lon) max(lon)])
ylim([min(lats) max(lats)])
hold off
%thres = max(max(abs(flipud(GT)-(datag.pot)./gamma)));
caxis([-thres thres])
%caxis([-1 1])
xlabel('Longitude [^o]','FontSize',20)
ylabel('Latitude [^o]','FontSize',20)
title(['WINTERC - XGM2016'],'FontSize',20)
ylabel(c,'m','FontSize',20) 
set(gca,'FontSize',20,'YDir','normal')

%% write data file to an ascii output file (uncomment lines)

% DATA_FILE = [reshape(data.grd.lon,[],1)   reshape(data.grd.lat,[],1)   reshape(flipud(GT),[],1) ...
%              reshape(datag.pot./gamma,[],1) reshape(data.pot./gamma,[],1)];
% 
% filename = fullfile('./','WINTERC-grav_geoid_test.txt');
% fid = fopen(filename, 'wt');
% fprintf(fid, '%s\t%s\t%s\t%s\t%s\n', 'Longitude (deg)','Latitude (deg) ','Geoid SHA (m)  ','Geoid XGM2016 (m)','Geoid GSH (mGal)');  % header
% fclose(fid);
% dlmwrite(filename,DATA_FILE,'delimiter','\t','precision','%15.8e','-append'); 

%%
function [rho0, alpha] = determineGradient(rho1,rho2,t1,t2,Re)
% function to calculate the density linear gradient inputs for 
a = (rho1-rho2)./((Re+t1)-(Re+t2));
a(isnan(a)) = 0;
rho0 = (rho1+rho2)./2;     % density at mean level
alpha = a./rho0 ;
alpha(rho0==0) = 0;
end