clear;
close all;
clc;

GSH_directory = '<ADD LOCAL INSTALL DIRECTORY FOR GSH PACKAGE>';
addpath([GSH_directory 'Tools/'])

%% initialize input settings for the benchmark

latLim =    [-89.75 89.75 0.5]; 
lonLim =    [0.25 359.75 0.5];

height =    10.0;
SHbounds =  [1 240];

Re = 6371000;
G = 6.6732e-11;
rho_mean = 5520;

M = 4*pi/3*Re^3*rho_mean;
GM = G*M;
g0 = GM/Re^2;

%% for analysis layers

latLimH =    [-89.75 89.75 0.5]; 
lonLimH =    [0.25 359.75 0.5]; 

lonH = lonLimH(1):lonLimH(3):lonLimH(2);
latH = fliplr(latLimH(1):latLimH(3):latLimH(2));
LonH = repmat(lonH,length(latH),1);
LatH = repmat(latH',1,length(lonH));

%% get geometry of layers

t1 = -56000.*ones(size(LonH));
t2 = -80000.*ones(size(LonH));

%% get densities from SHA code (from email correspondance)
 
dum1 = load('density_inputFile.txt');

dum1(dum1(:,2)==-90,:) = [];
dum1(dum1(:,1)==360,:) = [];

%% density file interpolation for forward gravity code

Fr1 = scatteredInterpolant(dum1(:,1),dum1(:,2),dum1(:,3),'linear','nearest');
rho1 = Fr1(LonH,LatH);

%% mass of the layer

res = 0.5;

phi0 = deg2rad(90-LatH-res/2);
phi1 = deg2rad(90-LatH+res/2);

theta0 = deg2rad(LonH-res/2);
theta1 = deg2rad(LonH+res/2);

%Mass_ratio = sum(sum(rho1 .* (cos(phi0)-cos(phi1)).*(theta1-theta0) .* (1./3 .*((Re+t1).^3-(Re+t2).^3))))./M

%% Construct input Model for the GSH code

Model = struct();

Model.number_of_layers = 1;
Model.name = 'ShellTest2_geoid';

% Additional variables
Model.GM = GM;
Model.Re = Re;
Model.geoid = 'none';
Model.nmax = 359;    

% Layer structure
Model.l1.bound = t1;
Model.l1.dens  = rho1;
Model.l2.bound = t2;

%% Do the GSH Analysis

[VA] = model_SH_analysis(Model);

%% Do the GSH Synthesis

disp('Performing the synthesis, this may take a while...')
[data1] = model_SH_synthesis(lonLim,latLim,height,SHbounds,VA,Model); 
disp('...Done!')
%% plot data

coast = load('coast.mat');
lon = data1.grd.lon;
lats = data1.grd.lat;

%% Laod the benchmark results from the SFA code by Zdenek Martinec

C = load('sha_results_test2.out');
MAP = gmt2matrix(C);

%% plot results

g_re = mean(mean((data1.pot./g0)./MAP));

nn = 30; % format size for plots

figure
imagesc(lon(1,:),lats(:,1),(MAP-(data1.pot./g0)));c=colorbar;colormap(parula) 
hold on
plot(coast.long,coast.lat,'k','LineWidth',1.5);plot(coast.long+360,coast.lat,'k','LineWidth',1.5);
hold off
axis([lonLim(1) lonLim(2) latLim(1) latLim(2)])
xlabel('Longitude [\circ]','FontSize',nn)
ylabel('Latitude [\circ]','FontSize',nn)
ylabel(c,'m','FontSize',nn)
thres = max(max(abs((MAP-(data1.pot./g0)))));
caxis([-thres thres])
set(gca,'YDir','normal','FontSize',nn)

figure
imagesc(lon(1,:),lats(:,1),flipud(rho1));c=colorbar;colormap(parula) 
hold on
plot(coast.long,coast.lat,'k','LineWidth',1.5);plot(coast.long+360,coast.lat,'k','LineWidth',1.5);
hold off
axis([lonLim(1) lonLim(2) latLim(1) latLim(2)])
xlabel('Longitude [\circ]','FontSize',nn)
ylabel('Latitude [\circ]','FontSize',nn)
ylabel(c,'kg/m^3','FontSize',nn)
set(gca,'YDir','normal','FontSize',nn)
